/*
 * Copyright (c) 2017 Oracle and/or its affiliates. All rights reserved. This
 * code is released under a tri EPL/GPL/LGPL license. You can use it,
 * redistribute it and/or modify it under the terms of the:
 *
 * Eclipse Public License version 1.0, or
 * GNU General Public License version 2, or
 * GNU Lesser General Public License version 2.1.
 */
package org.truffleruby.core.module;

import com.oracle.truffle.api.Assumption;
import com.oracle.truffle.api.CompilerDirectives.CompilationFinal;
import com.oracle.truffle.api.object.DynamicObject;
import org.truffleruby.RubyContext;
import org.truffleruby.language.LexicalScope;
import org.truffleruby.language.RubyConstant;

public class ConstantLookupResult {

    private final RubyConstant constant;
    @CompilationFinal(dimensions = 1) private final Assumption[] assumptions;

    public ConstantLookupResult(RubyConstant constant, Assumption... assumptions) {
        assert constant == null || !constant.isAutoloadingThread();
        this.constant = constant;
        this.assumptions = assumptions;
    }

    public boolean isFound() {
        return constant != null && !constant.isUndefined();
    }

    public boolean isDeprecated() {
        return constant != null && constant.isDeprecated();
    }

    public boolean isAutoload() {
        return constant != null && constant.isAutoload();
    }

    public boolean isVisibleTo(RubyContext context, LexicalScope lexicalScope, DynamicObject module) {
        return constant == null || constant.isVisibleTo(context, lexicalScope, module);
    }

    public RubyConstant getConstant() {
        return constant;
    }

    public Assumption[] getAssumptions() {
        return assumptions;
    }

}
