/*
 * Copyright (c) 2015, 2017 Oracle and/or its affiliates. All rights reserved. This
 * code is released under a tri EPL/GPL/LGPL license. You can use it,
 * redistribute it and/or modify it under the terms of the:
 *
 * Eclipse Public License version 1.0, or
 * GNU General Public License version 2, or
 * GNU Lesser General Public License version 2.1.
 */
package org.truffleruby.language.backtrace;

import java.util.ArrayList;
import java.util.List;

public class BacktraceInterleaver {

    public static List<String> interleave(List<String> rubyBacktrace, StackTraceElement[] javaStacktrace, int omitted) {
        final List<String> interleaved = new ArrayList<>();

        int javaIndex = 0;
        int rubyIndex = 0;
        int toOmit = omitted;

        while (javaIndex < javaStacktrace.length || rubyIndex < rubyBacktrace.size()) {
            if (javaIndex >= javaStacktrace.length || (isCallBoundary(javaStacktrace[javaIndex]) && rubyIndex < rubyBacktrace.size())) {
                if (toOmit == 0) {
                    interleaved.add(rubyBacktrace.get(rubyIndex));
                    rubyIndex++;
                } else {
                    toOmit--;
                }
            }

            if (javaIndex < javaStacktrace.length) {
                interleaved.add(String.format("\t\t%s", javaStacktrace[javaIndex]));

                if (isIntoRuby(javaStacktrace, javaIndex)) {
                    interleaved.add("\t\t\tforeign call goes into Ruby");
                }

                if (isForeignCall(javaStacktrace[javaIndex])) {
                    interleaved.add("\t\t\tforeign call being made");
                }

                javaIndex++;
            }
        }

        return interleaved;
    }

    private static boolean isCallBoundary(StackTraceElement element) {
        return element.toString().startsWith("com.oracle.graal.truffle.OptimizedCallTarget.callProxy")
                || element.toString().startsWith("com.oracle.truffle.api.impl.DefaultCallTarget.call");
    }

    private static boolean isIntoRuby(StackTraceElement[] elements, int index) {
        if (index + 1 >= elements.length) {
            return false;
        }

        return elements[index].toString().startsWith("org.truffleruby.interop.RubyMessageResolutionForeign")
                && !elements[index + 1].toString().startsWith("org.truffleruby.interop.RubyMessageResolutionForeign");
    }

    private static boolean isForeignCall(StackTraceElement element) {
        return element.toString().startsWith("com.oracle.truffle.api.interop.ForeignAccess.send");
    }

}
